@@ -1,6 +1,6 @@
-#' Faster operations on rasters (DEPRECATED as `terra::mask` is fast)
+#' Faster operations on rasters (DEPRECATED because `terra::mask` is fast)
 #'
-#' This alternative to `raster::mask` is included here.
+#' Deprecated. Use [maskTo()].
 #'
 #' @param x  A `Raster*` object.
 #'
@@ -13,6 +13,9 @@
 #'           float will be passed as the exact number of cores to be used.
 #' @param skipDeprecastedMsg Logical. If `TRUE`, then the message about this function
 #'   being deprecated will be suppressed.
+#' @param useGDAL Deprecated. Logical or `"force"`. This is defunct; internals now can use
+#'     `terra` if `options("reproducible.useTerra" = TRUE)`, which is not (yet)
+#'     the default.
 #'
 #' @param ... Currently unused.
 #'
@@ -22,12 +25,9 @@
 #' @author Eliot McIntire
 #' @export
 #' @inheritParams Cache
-#' @inheritParams projectInputs.Raster
-#' @importFrom raster crop crs extract mask nlayers raster stack tmpDir
-#' @importFrom raster xmin xmax ymin ymax fromDisk setMinMax
-#' @importFrom sp SpatialPolygonsDataFrame spTransform
+#' @inheritParams projectInputs
 #'
-fastMask <- function(x, y, cores = NULL, useGDAL = getOption("reproducible.useGDAL", FALSE),
+fastMask <- function(x, y, cores = NULL, useGDAL = FALSE,
                      verbose = getOption("reproducible.verbose", 1), ..., skipDeprecastedMsg = FALSE) {
   if (!skipDeprecastedMsg)
     .Deprecated("mask", "terra", "fastMask is deprecated; using maskTo and terra")
@@ -35,62 +35,27 @@ fastMask <- function(x, y, cores = NULL, useGDAL = getOption("reproducible.useGD
   maskTo(from = x, maskTo = y, touches = isTRUE(touches))
 }
 
-#' @importFrom raster tmpDir
 bigRastersTmpFolder <- function() checkPath(tempdir2(sub = "bigRasters"), create = TRUE)
 
 bigRastersTmpFile <- function() file.path(bigRastersTmpFolder(), "bigRasInput.tif")
 
 
-attemptGDAL <- function(x, useGDAL = getOption("reproducible.useGDAL", FALSE),
-                        verbose = getOption("reproducible.verbose", 1)) {
-  attemptGDAL <- FALSE
-  if (is(x, "Raster")) {
-    crsIsNA <- is.na(.crs(x))
-    cpim <- canProcessInMemory(x, 3)
-    isTRUEuseGDAL <- isTRUE(useGDAL)
-    forceGDAL <- identical(useGDAL, "force")
-    shouldUseGDAL <- (!cpim && isTRUEuseGDAL || forceGDAL)
-    attemptGDAL <- if (shouldUseGDAL && !crsIsNA) {
-      TRUE
-    } else {
-      if (crsIsNA && shouldUseGDAL)
-        messagePrepInputs("      Can't use GDAL because crs is NA", verbose = verbose)
-      if (cpim && isTRUEuseGDAL)
-        messagePrepInputs("      useGDAL is TRUE, but problem is small enough for RAM; skipping GDAL; ",
-                          "useGDAL = 'force' to override", verbose = verbose)
 
-      FALSE
-    }
-  }
-  attemptGDAL
-}
-
-maskWithRasterNAs <- function(x, y) {
-  origColors <- checkColors(x)
-  if (canProcessInMemory(x, 3) && fromDisk(x))
-    x[] <- x[]
-  x <- rebuildColors(x, origColors)
-  m <- which(is.na(y[]))
-  x[m] <- NA
-  x
-}
-
-#' @importFrom raster maxValue minValue
 checkColors <- function(x) {
   origColors <- .getColors(x)
-  origMaxValue <- maxValue(x)
-  origMinValue <- minValue(x)
+  origMaxValue <- maxFn(x)
+  origMinValue <- minFn(x)
   list(origColors = origColors[[1]], origMinValue = origMinValue, origMaxValue = origMaxValue)
 }
 
 rebuildColors <- function(x, origColors) {
-  if (isTRUE(all(origColors$origMinValue != minValue(x)) || all(origColors$origMaxValue != maxValue(x)) ||
+  if (isTRUE(all(origColors$origMinValue != minFn(x)) || all(origColors$origMaxValue != maxFn(x)) ||
              !identical(.getColors(x)[[1]], origColors$origColors))) {
     colorSequences <- unlist(lapply(seq(length(origColors$origMinValue)), function(ind) {
       origColors$origMinValue[ind]:origColors$origMaxValue[ind]
     }))
     if (isTRUE(length(origColors$origColors) == length(colorSequences))) {
-      newSeq <- minValue(x):maxValue(x)
+      newSeq <- minFn(x):maxFn(x)
       oldSeq <- origColors$origMinValue:origColors$origMaxValue
       whFromOld <- match(newSeq, oldSeq)
       x@legend@colortable <- origColors$origColors[whFromOld]
@@ -101,11 +66,15 @@ rebuildColors <- function(x, origColors) {
 
 
 .getColors <- function(object) {
-  nams <- names(object)
-  cols <- lapply(nams, function(x) {
-    as.character(object[[x]]@legend@colortable)
-  })
-  names(cols) <- nams
+  if (is(object, "SpatRaster")) {
+    cols <- terra::coltab(object)
+  } else {
+    nams <- names(object)
+    cols <- lapply(nams, function(x) {
+      as.character(object[[x]]@legend@colortable)
+    })
+    names(cols) <- nams
+  }
   return(cols)
 }
 
