@@ -1,9 +1,4 @@
 test_that("checkPath: normPath consistency", {
-  # testInitOut <- testInit("raster", tmpFileExt = c(".tif", ".grd"))
-  # on.exit({
-  #   testOnExit(testInitOut)
-  # }, add = TRUE)
-
   # Use the following here instead of above because it fails on Mac without this.
   tmpdir <- tempdir2("test_normPath")
   tmpdir <- normalizePath(tmpdir, winslash = "/", mustWork = FALSE)
@@ -38,21 +33,11 @@ test_that("checkPath: normPath consistency", {
 })
 
 test_that("checkPath: checkPath consistency", {
-  testInitOut <- testInit("raster", tmpFileExt = c(".tif", ".grd"))
+  testInitOut <- testInit()
   on.exit({
     testOnExit(testInitOut)
   }, add = TRUE)
 
-  # don't use checkPath here because we are testing checkPath
-  # currdir <- getwd()
-  # tmpdir <- tempdir2("test_checkPath")
-  #
-  # on.exit({
-  #   setwd(currdir)
-  #   unlink(tmpdir, recursive = TRUE)
-  # }, add = TRUE) # nolint
-  # setwd(tmpdir)
-
   dir.create("aaa/zzz", recursive = TRUE, showWarnings = FALSE)
   paths <- list("./aaa/zzz",
                 "./aaa/zzz/",
@@ -83,3 +68,4 @@ test_that("checkPath: checkPath consistency", {
   expect_message(a <- checkPath(f1), "is an existing file")
 
 })
+
