@@ -1,17 +1,17 @@
 test_that("test Copy", {
-  testInitOut <- testInit(c("raster", "data.table"), tmpFileExt = ".tif")
+  testInitOut <- testInit(c("terra", "data.table"), tmpFileExt = ".tif")
   on.exit({
     testOnExit(testInitOut)
   }, add = TRUE)
 
-  ras <- raster(extent(0, 10, 0, 10), vals = 1)
+  ras <- terra::rast(terra::ext(0, 10, 0, 10), vals = 1)
   ras <- suppressWarningsSpecific(falseWarnings = proj6Warn,
                                   writeRaster(ras, filename = tmpfile, overwrite = TRUE))
   # This will make hardlink
   ras2 <- suppressWarningsSpecific(Copy(ras, tmpdir), "NOT UPDATED FOR PROJ >= 6")
 
   expect_true(all.equal(ras2[], ras[]))
-  expect_false(filename(ras2) == filename(ras))
+  expect_false(Filenames(ras2) == Filenames(ras))
 
   dt <- data.table(a = 1:2, b = rev(LETTERS[1:2]))
   tmpdir <- normPath(tempdir2("ras2"))
@@ -21,8 +21,8 @@ test_that("test Copy", {
 
   # same content
   expect_true(all(unlist(lapply(seq_along(li), function(i) {
-    if (is(li[[i]], "Raster")) {
-       all.equal(getValues(li[[i]]), raster::getValues(li2[[i]]))
+    if (is(li[[i]], "SpatRaster")) {
+       all.equal(values2(li[[i]]), values2(li2[[i]]))
      } else {
       all.equal(li[[i]], li2[[i]])
     }
@@ -30,7 +30,7 @@ test_that("test Copy", {
 
   # different filenames for Rasters
   expect_false(all(unlist(lapply(seq_along(li)[-1], function(i) {
-    isTRUE(all.equal(filename(li[[i]]), filename(li2[[i]])))
+    isTRUE(all.equal(Filenames(li[[i]]), Filenames(li2[[i]])))
   }))))
 
   # data.table
@@ -46,7 +46,7 @@ test_that("test Copy", {
   li2 <- Copy(li, tmpdir)
 
   expect_true(all(unlist(lapply(names(li), function(i) {
-    if (is(li[[i]], "Raster")) {
+    if (is(li[[i]], "SpatRaster")) {
       all.equal(li[[i]][], li2[[i]][])
     } else {
       all.equal(li[[i]], li2[[i]])
@@ -55,7 +55,7 @@ test_that("test Copy", {
 
   # different filenames for Rasters
   expect_false(all(unlist(lapply(names(li)[-1], function(i) {
-    isTRUE(all.equal(filename(li[[i]]), filename(li2[[i]])))
+    isTRUE(all.equal(Filenames(li[[i]]), Filenames(li2[[i]])))
   }))))
 
   # data.table
@@ -74,7 +74,7 @@ test_that("test Copy", {
   liEnv2 <- Copy(liEnv, tmpdir)
 
   expect_true(all(unlist(lapply(names(liEnv[["env"]]), function(i) {
-    if (is(li[[i]], "Raster")) {
+    if (is(li[[i]], "SpatRaster")) {
       all.equal(liEnv[["env"]][[i]][], liEnv2[["env"]][[i]][])
     } else {
       all.equal(liEnv[["env"]][[i]], liEnv2[["env"]][[i]])
@@ -83,7 +83,7 @@ test_that("test Copy", {
 
   # different filenames for Rasters
   expect_false(all(unlist(lapply(names(liEnv[["env"]])[-1], function(i) {
-    isTRUE(all.equal(filename(liEnv[["env"]][[i]]), filename(liEnv2[["env"]][[i]])))
+    isTRUE(all.equal(Filenames(liEnv[["env"]][[i]]), Filenames(liEnv2[["env"]][[i]])))
   }))))
 
   # data.table
@@ -93,7 +93,7 @@ test_that("test Copy", {
   ###################
   # This is an aside on testing whether the hard link can be messed with by overwrite
   fn <- Filenames(ras)
-  ras3 <- raster(extent(0, 10, 0, 10), vals = 2)
+  ras3 <- terra::rast(terra::ext(0, 10, 0, 10), vals = 2)
   ras3 <- suppressWarningsSpecific(writeRaster(ras3, filename = tmpfile, overwrite = TRUE),
                                    "NOT UPDATED FOR PROJ >= 6")# overwrite the original
   # The hardlink is not affected by "overwrite = TRUE" -- it is not by filename, but by file location
