@@ -1,7 +1,5 @@
 test_that("test devMode", {
-  testInitOut <- testInit("raster", tmpFileExt = c(".tif", ".grd"),
-                          opts = list("reproducible.useCache" = "devMode",
-                                      "reproducible.useNewDigestAlgorithm" = 2))
+  testInitOut <- testInit(opts = list("reproducible.useCache" = "devMode"))
   on.exit({
     testOnExit(testInitOut)
   }, add = TRUE)
